@@ -1,7 +1,7 @@
 /* affs.c - Amiga Fast FileSystem.  */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2005  Free Software Foundation, Inc.
+ *  Copyright (C) 2005,2006  Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -150,9 +150,9 @@ grub_affs_read_block (grub_fshelp_node_t node, int fileblock)
    POS.  Return the amount of read bytes in READ.  */
 static grub_ssize_t
 grub_affs_read_file (grub_fshelp_node_t node,
-		     void (*read_hook) (unsigned long sector,
+		     void (*read_hook) (grub_disk_addr_t sector,
 					unsigned offset, unsigned length),
-		     int pos, unsigned int len, char *buf)
+		     int pos, grub_size_t len, char *buf)
 {
   return grub_fshelp_read_file (node->data->disk, node, read_hook,
 				pos, len, buf, grub_affs_read_block,
@@ -182,7 +182,7 @@ grub_affs_mount (grub_disk_t disk)
     goto fail;
 
   /* Make sure this is an affs filesystem.  */
-  if (grub_strncmp (data->bblock.type, "DOS", 3))
+  if (grub_strncmp ((char *) (data->bblock.type), "DOS", 3))
     {
       grub_error (GRUB_ERR_BAD_FS, "not an affs filesystem");
       goto fail;
@@ -364,7 +364,7 @@ grub_affs_iterate_dir (grub_fshelp_node_t dir,
 	  else
 	    type = GRUB_FSHELP_UNKNOWN;
 
-	  if (grub_affs_create_node (file.name, next,
+	  if (grub_affs_create_node ((char *) (file.name), next,
 				     grub_be_to_cpu32 (file.size), type))
 	    return 1;
 
@@ -439,7 +439,7 @@ grub_affs_close (grub_file_t file)
 
 /* Read LEN bytes data from FILE into BUF.  */
 static grub_ssize_t
-grub_affs_read (grub_file_t file, char *buf, grub_ssize_t len)
+grub_affs_read (grub_file_t file, char *buf, grub_size_t len)
 {
   struct grub_affs_data *data = 
     (struct grub_affs_data *) file->data;
@@ -527,7 +527,7 @@ grub_affs_label (grub_device_t device, char **label)
       if (grub_errno)
 	return 0;
 
-      *label = grub_strndup (file.name, file.namelen);
+      *label = grub_strndup ((char *) (file.name), file.namelen);
     }
   else
     *label = 0;
