@@ -39,7 +39,7 @@ static const grub_addr_t grub_heap_start = 0x4000;
 static grub_addr_t grub_heap_len;
 
 void
-abort (void)
+grub_exit (void)
 {
   /* Trap to Open Firmware.  */
   asm ("trap");
@@ -62,13 +62,17 @@ grub_translate_ieee1275_path (char *filepath)
     }
 }
 
-static void
-grub_set_prefix (void)
+void
+grub_machine_set_prefix (void)
 {
   char bootpath[64]; /* XXX check length */
   char *filename;
   char *prefix;
 
+  if (grub_env_get ("prefix"))
+    /* We already set prefix in grub_machine_init().  */
+    return;
+
   if (grub_ieee1275_get_property (grub_ieee1275_chosen, "bootpath", &bootpath,
 				  sizeof (bootpath), 0))
     {
@@ -112,7 +116,6 @@ void
 grub_machine_init (void)
 {
   char args[256];
-  grub_ieee1275_phandle_t chosen;
   int actual;
   extern char _start;
 
@@ -126,17 +129,14 @@ grub_machine_init (void)
     {
       grub_printf ("Failed to claim heap at 0x%x, len 0x%x\n", grub_heap_start,
 		   grub_heap_len);
-      abort ();
+      grub_abort ();
     }
   grub_mm_init_region ((void *) grub_heap_start, grub_heap_len);
 
-  grub_set_prefix ();
-
   grub_ofdisk_init ();
 
   /* Process commandline.  */
-  grub_ieee1275_finddevice ("/chosen", &chosen);
-  if (grub_ieee1275_get_property (chosen, "bootargs", &args,
+  if (grub_ieee1275_get_property (grub_ieee1275_chosen, "bootargs", &args,
 				  sizeof args, &actual) == 0
       && actual > 1)
     {
@@ -177,19 +177,12 @@ grub_machine_fini (void)
   grub_console_fini ();
 }
 
-void
-grub_stop (void)
-{
-  for (;;);
-}
-
 grub_uint32_t
 grub_get_rtc (void)
 {
-  grub_uint32_t msecs;
+  grub_uint32_t msecs = 0;
 
-  if (grub_ieee1275_milliseconds (&msecs))
-    return 0;
+  grub_ieee1275_milliseconds (&msecs);
 
   return msecs;
 }
