@@ -127,7 +127,8 @@ grub_multiboot_load_elf32 (grub_file_t file, void *buffer)
 	    return grub_error (GRUB_ERR_BAD_OS,
 			       "segment doesn't fit in memory reserved for the OS");
 
-          if (grub_file_seek (file, phdr->p_offset) == -1)
+          if (grub_file_seek (file, (grub_off_t) phdr->p_offset)
+	      == (grub_off_t) -1)
 	    return grub_error (GRUB_ERR_BAD_OS,
 			       "invalid offset in program header");
 	  
@@ -201,7 +202,8 @@ grub_multiboot_load_elf64 (grub_file_t file, void *buffer)
 	    return grub_error (GRUB_ERR_BAD_OS,
 			       "segment doesn't fit in memory reserved for the OS");
 	  
-	  if (grub_file_seek (file, phdr->p_offset) == -1)
+	  if (grub_file_seek (file, (grub_off_t) phdr->p_offset)
+	      == (grub_off_t) -1)
 	    return grub_error (GRUB_ERR_BAD_OS,
 			       "invalid offset in program header");
 
@@ -326,7 +328,7 @@ grub_rescue_cmd_multiboot (int argc, char *argv[])
   mbi->flags |= GRUB_MB_INFO_BOOT_LOADER_NAME;
   mbi->boot_loader_name = (grub_uint32_t) grub_strdup (PACKAGE_STRING);
 
-  grub_loader_set (grub_multiboot_boot, grub_multiboot_unload);
+  grub_loader_set (grub_multiboot_boot, grub_multiboot_unload, 1);
 
  fail:
   if (file)
@@ -435,7 +437,7 @@ grub_rescue_cmd_module  (int argc, char *argv[])
 }
 
 
-GRUB_MOD_INIT
+GRUB_MOD_INIT(multiboot)
 {
   grub_rescue_register_command ("multiboot", grub_rescue_cmd_multiboot,
 				"load a multiboot kernel");
@@ -444,7 +446,7 @@ GRUB_MOD_INIT
   my_mod = mod;
 }
 
-GRUB_MOD_FINI
+GRUB_MOD_FINI(multiboot)
 {
   grub_rescue_unregister_command ("multiboot");
   grub_rescue_unregister_command ("module");
